### prepare the root by creating basic devices and directories
prep: check
	@if test ! -f $(MACHROOT)/state/prep; then \
          echo "Preparing root ..."; \
          make lock; \
          mkdir -p $(ROOT)/var/lib/rpm; \
          chmod g+s $(ROOT)/var/lib/rpm; \
          mkdir -p $(ROOT)/dev; \
          $(MKNOD) $(ROOT)/dev/null c 1 3; \
          mkdir -p $(ROOT)/etc; \
	  cp $(MACHROOT)/etc/resolv.conf $(ROOT)/etc; \
	  cp $(MACHROOT)/etc/hosts $(ROOT)/etc; \
	  touch $(ROOT)/etc/mtab; \
	  touch $(ROOT)/etc/fstab; \
          mkdir -p $(ROOT)/tmp $(ROOT)/var/tmp; \
          rpm --root $(ROOT) --initdb; \
          touch $(MACHROOT)/state/prep; \
          make unlock; \
	else \
	  echo "prep already made."; \
	fi

#for pkg in `cat manifest/base | perl -pe 's@^@../../packages/@' | perl -pe 's/\n/ /g'`; do \
          echo "installing $$pkg"; \
	  $(RPM) --root $(ROOT) -Uhv $$pkg --nodeps || exit 1; \
          done; \### install all packages from the dist's manifest/base dir if not done yet

base: prep
	@if test ! -f $(MACHROOT)/state/base; then \
	  echo "Installing base ..."; \
	  make lock; \
	  $(RPM) --root $(ROOT) -Uhv `cat manifest/base | perl -pe 's@^@../../packages/@' | perl -pe 's/\n/ /g'`  --nodeps --noorder || exit 1; \
	  touch $(MACHROOT)/state/base; \
	  echo "DISTRO=$(DIST)" > $(MACHROOT)/state/dist.mk; \
	  make unlock; \
	else \
	  echo "base already made."; \
	fi

### install all packages from apt if apt has not been made yet
apt: base
	@if test ! -f $(MACHROOT)/state/apt; then \
	  echo "Installing apt ..."; \
	  make lock; \
	  $(RPM) --root $(ROOT) -Uhv `cat manifest/apt | perl -pe 's@^@../../packages/@' | perl -pe 's/\n/ /g'`  --nodeps; \
	  touch $(MACHROOT)/state/apt; \
	  make unlock; \
	else \
	  echo "apt already made."; \
	fi

### install all stuff necessary for rpm -tb to work
# FIXME: the ugly workaround to create rpmmacros is because I can't figure
# out a way to make a redirect happen in the subshell, inside the chroot
# ie $(CHROOT) $(ROOT) echo "%_topdir /usr/src/rpm" > $(ROOT)/root/.rpmmacros
rpm-build: root
	@if test ! -f $(MACHROOT)/state/rpm-build; then \
	  echo "Installing rpm-build ..."; \
	  make lock; \
	  $(CHROOT) $(ROOT) apt-get install -y rpm-devel rpm-build make gcc tar gzip patch unzip diffutils $(RPM_BUILD); \
	  echo "%_topdir /usr/src/rpm" > $(ROOT)/tmp/macros; \
	  $(CHROOT) $(ROOT) mv /tmp/macros /etc/rpm; \
          for a in RPMS SRPMS SOURCES SPECS BUILD; do \
	    $(CHROOT) $(ROOT) mkdir -p /usr/src/rpm/$$a; \
          done; \
	  $(RPM) --root $(ROOT) -qa > $(MACHROOT)/state/rpm-build; \
	  make unlock; \
	else \
          make verify-rpm-build; \
	fi

### verify that the state is correct
### this basically checks at the file for a package list
### and compares it to the list of packages in the root
### if it's not the same, it removes the surplus packages
# FIXME: generalize this using Makefile rules
# FIXME: make this use proper temp files for intermediary files
# FIXME: add another check after rpm remove to see if it's ok

### this is the closest we have to subroutines in make
define rpm-root-list
rpm --root $(ROOT) -qa
endef

verify-rpm-build:
	@echo "Verifying rpm-build ..."
	@if $(rpm-root-list) | diff - $(MACHROOT)/state/rpm-build > /dev/null 2>&1; then \
	  echo "rpm-build is ok."; \
	else \
	  echo "rpm-build requires some removals."; \
          REMOVE=`$(rpm-root-list) | grep -v -f $(MACHROOT)/state/rpm-build - | tr '\n' ' '` && \
	  echo "removing $$REMOVE"; \
	  $(CHROOT) $(ROOT) rpm -e $$REMOVE || echo "Error returning to base state !"; \
	fi

### do the final stuff to make the root ready
### mount the apt-archives and proc binded into the root
root: apt sources.list
	@if test ! -f $(MACHROOT)/state/ready; then \
          echo "Readying root ..."; \
          make lock; \
	  make umount; \
	  $(CHROOT) $(ROOT) rm -rf /var/cache/apt/archives/*; \
	  make mount-apt-archives; \
	  make mount-proc; \
	  $(CHROOT) $(ROOT) rm -rf /var/lib/rpm/__*; \
	  $(CHROOT) $(ROOT) apt-get update && \
	  $(CHROOT) $(ROOT) apt-get install -f -y && \
          touch $(MACHROOT)/state/ready && \
          make unlock; \
        else \
          echo "Root ready already."; \
        fi

mount-proc:
	  @$(MOUNT) -t proc proc $(ROOT)/proc
	  @echo "$(ROOT)/proc" >> $(MACHROOT)/state/mount

mount-apt-archives:
	  @$(MOUNT) -o bind $(MACHROOT)/apt-archives $(ROOT)/var/cache/apt/archives
	  @echo "$(ROOT)/var/cache/apt/archives" >> $(MACHROOT)/state/mount

### perform various checks on our sanity
check: check-manifest check-dirs check-etc
	@echo "Pre-flight check ok."

### import GPG keys, note that "rpm --import" requires RPM >= 4.1
import-keys: prep
	@if test -f keys; then \
	  make lock; \
	  pwd=`pwd`; \
	  cd $(ROOT)/tmp; \
	  mkdir -p -m 0700 $(ROOT)/root/.gnupg; \
	  touch $(ROOT)/root/.gnupg/options; \
	  for foo in `cat $$pwd/keys`; do \
	    rpm=$(MACHROOT)/packages/`echo $$foo | perl -pe 's/:.*//'`; \
	    pat=`echo $$foo | perl -pe 's/.*://'`; \
	    key=`rpm2cpio $$rpm | cpio -ivdu --quiet $$pat 2>&1`; \
	    gpg --homedir $(ROOT)/root/.gnupg -q --import $$key; \
	    $(RPM) --root $(ROOT) --import $$key; \
	  done; \
	  cd $$pwd; \
	  rm -rf $(ROOT)/tmp/*; \
	  make unlock; \
	fi

### try to make each file mentioned in the manifests into the packages dir
make-manifest:
	@for a in manifest/*; do \
	  if test -f $$a; then \
	    for b in `cat $$a | perl -pe 's/\n/ /g'`; do \
              if test ! -f $(MACHROOT)/packages/$$b; then \
		if test ! -z $(CDROM); then \
		  cp $(CDROM)/`basename $$b` $(MACHROOT)/packages/$$b; \
		else \
		  cd $(MACHROOT)/packages && make $$b; \
		fi; \
              fi; \
            done; \
	  fi; \
	done

### check if the packages listed in the manifest files are available
check-manifest:
	@for a in manifest/*; do \
	  if test -f $$a; then \
	    for b in `cat $$a | perl -pe 's/\n/ /g'`; do \
	      if test ! -f $(MACHROOT)/packages/$$b; then \
	        echo "$$b not found in"; \
	        echo "    $(MACHROOT)/packages (from $$a)"; \
		FAILED=true; \
	      fi; \
	    done; \
	    if test "x$$FAILED" = "xtrue"; then \
	      echo "Packages not found.  Try running \"make make-manifest\""; \
              exit 1; \
            fi; \
	  fi; \
	done

check-dirs:
	@mkdir -p $(MACHROOT)/apt-archives/partial
	@mkdir -p $(ROOT)

check-etc:
	@if test ! -f $(MACHROOT)/etc/resolv.conf; then \
	  echo "Please create $(MACHROOT)/etc/resolv.conf !"; \
	  exit 1; \
	fi
	@if test ! -f $(MACHROOT)/etc/hosts; then \
	  echo "Please create $(MACHROOT)/etc/hosts !"; \
	  exit 1; \
	fi

### create an e2fs file containing everything in root (with mounted dirs
### unmounted
e2fs: root 
	@if test `id -u` -ne "0"; then \
	  echo "You must run this as root for it to work well."; exit 1; \
        else \
          echo Creating $(MACHROOT)/e2fs file system && \
	  make lock umount && \
	  dd if=/dev/zero of=$(MACHROOT)/e2fs seek=`$(CHROOT) $(ROOT) du -s -B 512K / | cut -f1` count=1 bs=1M && \
	  /sbin/mke2fs -F $(MACHROOT)/e2fs && \
	  mkdir /tmp/e2fs && \
	  $(MOUNT) -o loop $(MACHROOT)/e2fs /tmp/e2fs && \
	  cp -prv $(ROOT)/* /tmp/e2fs && \
	  $(UMOUNT) $(MACHROOT)/e2fs && \
	  make mount-proc mount-apt-archives && \
	  rmdir /tmp/e2fs && \
          echo "$(MACHROOT)/e2fs created"; \
        fi


